/*
 * Copyright (c) 2005, Adam Gleave
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the 
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the copyright holders nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT OWNER AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef P_IOUNIX_H
#define P_IOUNIX_H

#include "arr.h"

#define P_IOP_RET_INTFAIL -2
#define P_IOP_RET_EXTFAIL -1

#define P_IOP_RET_PARTREAD 0
#define P_IOP_RET_READ 1

#define P_IOP_RET_PARTWROTE 0
#define P_IOP_RET_WROTE 1

/* PURPOSE
 * To append p_size bytes from p_fd to p_buf.
 * PARAMATERS
 * p_buf        - Pointer to p_arr structure.
 * p_fd         - File descriptor to read from.
 * p_size       - Number of bytes to read.
 * RETURN VALUES
 * -2 (P_IOP_RET_INTFAIL) is returned on non-IO failure, -1 (P_IOP_RET_EXTFAIL)
 * on IO failure. On success, number of bytes read is returned. */

ssize_t p_io_read(p_arr *p_buf, int p_fd, size_t p_size);

/* PURPOSE
 * To do it's best to append *p_size bytes from p_fd to p_buf.
 * PARAMATERS
 * p_buf        - Pointer to p_arr structure.
 * p_fd         - File descriptor to read from.
 * p_size       - Pointer to number of bytes to read.
 * RETURN VALUES
 * -2 (P_IOP_RET_INTFAIL) on internal failure, -1 (P_IOP_RET_EXTFAIL)
 * on external failure, 0 (P_IOP_RET_PARTREAD) on partial read and
 * 1 (P_IOP_RET_READ) on complete read. *p_size is set to number of
 * bytes read. */

ssize_t p_io_read_all(p_arr *p_buf, int p_fd, size_t *p_size);

/* PURPOSE
 * To write p_size bytes from p_buf into p_fd.
 * PARAMATERS
 * p_fd         - File descriptor to write to.
 * p_buf        - Pointer to p_arr structure.
 * p_size       - Number of bytes to write.
 * RETURN VALUES
 * -2 (P_IOP_RET_INTFAIL) is returned on non-IO failure, -1 (P_IOP_RET_EXTFAIL)
 * on IO failure. On success, number of bytes written is returned. */

ssize_t p_io_write(int p_fd, p_arr *p_buf, size_t p_size);

/* PURPOSE
 * To write *p_size bytes from p_buf into p_fd, doing it's best to send all
 * of *p_size.
 * PARAMATERS
 * p_fd        - File descriptor to write to.
 * p_buf       - Pointer to p_arr structure.
 * p_size      - Pointer to number of bytes to write.
 * RETURN VALUES
 * -2 (P_IOP_RET_INTFAIL) on internal failure, -1 (P_IOP_RET_EXTFAIL) on
 * external failure, 0 (P_IOP_RET_PARTWROTE) on partial send and
 * 1 (P_IOP_RET_WROTE) on complete send. *p_size is set to number of bytes
 * written. */

int p_io_write_all(int p_fd, p_arr *p_buf, size_t *p_size);

#endif /* P_IOUNIX_H */
