/*
 * Copyright (c) 2005, Adam Gleave
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the 
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the copyright holders nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT OWNER AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef P_PRIVIOUNIXNET_H
#define P_PRIVIOUNIXNET_H

/* P_PRIVATE_IOUNIXNET_WRITEALLBEG() and P_PRIVATE_IOUNIXNET_WRITEALLEND()
 * PURPOSE
 * To try it's best to write *p_size bytes to p_fd from p_buf.
 * PARAMATERS
 * p_fd         - Destination file decriptor.
 * p_buf        - Source buffer.
 * p_wrote      - Name of size_t variable to use as amount written.
 * p_size       - Name of size_t * variable to use as size to write.
 * EXPECTED VARIABLES
 * ret          - ssize_t.
 * buf          - const unsigned char *.
 * RETURN VALUES
 * It does not return.
 * NOTES
 * Immediately after the P_PRIVATE_IOUNIXNET_WRITEALLBEG() macro you must
 * place ret = writecmd. After this, place P_PRIVATE_IOUNIXNET_WRITEALLEND().
 * writecmd must be a function that returns -1 on error and number of bytes
 * written on success. 
 * 
 * Failure to comply with the above will result in chaos ensuing. If chaos
 * does not ensue, you did something wrong. */

#define P_PRIVATE_IOUNIXNET_WRITEALLBEG(p_fd, p_buf, p_wrote, p_size)      \
  p_wrote = 0;                                                             \
  if ((buf = p_buf_read_pos(p_buf, p_wrote, *p_size)) == NULL) {           \
    /* We can't read the buffer. */                                        \
    *p_size = p_wrote;                                                     \
    return -2; /* -2 = INTFAIL */                                          \
  }                                                                        \
  while (1) {                                                              \

#define P_PRIVATE_IOUNIXNET_WRITEALLEND(p_fd, p_buf, p_wrote, p_size)      \
    if (ret < 0) {                                                         \
      /* Error sending. */                                                 \
      *p_size = p_wrote;                                                   \
      return -1; /* -1 = EXTFAIL */                                        \
    } else if (ret == 0) {                                                 \
      /* We're not sending anymore. */                                     \
      *p_size = p_wrote;                                                   \
      return 0; /* 0 = PARTSENT */                                         \
    }                                                                      \
    p_wrote += ret;                                                        \
    *p_size -= ret;                                                        \
                                                                           \
    if (*p_size == 0) {                                                    \
      /* We've written everything - great! */                              \
      *p_size = p_wrote;                                                   \
      return 1; /* 1 = SENT */                                             \
    }                                                                      \
                                                                           \
    buf += ret;                                                            \
  } /* END: while (1) */

#endif /* P_PRIVIOUNIXNET_H */
