/*
 * Copyright (c) 2005, Adam Gleave
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the 
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the copyright holders nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT OWNER AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#define LBUG(p_text) \
  f_lbug(p_text, __LINE__)

#define LDIE(p_text) \
  f_ldie(p_text, __LINE__)

#define LDIEPRINTF(p_format, p_valist) \
  f_ldieprintf(p_format, p_valist, __LINE__)

#define LBUGPRINTF(p_format, p_valist) \
  f_lbugprintf(__LINE__, p_format, p_valist)

#define READCMP(p_buf, p_text, p_len) \
  if ((p = p_buf_read(p_buf)) == NULL) \
    LBUG("main: p_buf_read: returned NULL."); \
  if (memcmp(p, p_text, p_len) != 0) \
    LBUG("main: memcmp: arrays don't match.")

#define USEDCHECK(p_buf, p_used) \
  if (p_buf_get_used(p_buf) != p_used) \
    LBUG("main: p_buf_get_used: value not equal.")

/* f_ldie()
 * PURPOSE
 * To terminate execution of the program with message p_text.
 * It indicates that source code line p_line called it.
 * PARAMATERS
 * p_text         - Pointer to array of characters.
 * p_line         - Line number. 
 * RETURN VALUES
 * It does not return. */
 
void f_ldie(char *p_text, int p_line);

/* f_lbug()
 * PURPOSE
 * To terminate execution of the program with message p_text. It indicates
 * that source code line p_line called it and that it is a bug condition.
 * PARAMATERS
 * p_text         - Pointer to array of characters.
 * p_line         - Line number. 
 * RETURN VALUES
 * It does not return. */

void f_lbug(char *p_text, int p_line);

/* f_lbugprintf()
 * PURPOSE
 * To terminate execution of the program with a formatted message. It
 * indcates that source coede line p_line called it and that it is a bug
 * condition.
 * p_line         - Line number.
 * p_format       - Format string.
 * RETURN VALUES
 * It does not return. */

void f_lbugprintf(int p_line, const char *p_format, ...);
